{
LiveQuery Delphi Component

Supports editing of SQL server result sets through
the use of temporary views.  This allows any result
set to be updated that would be editable had the
user created it as a view on the back-end.  Any updates
that would be supported by the back-end against views
are therefore supported.

Written by Ken Henderson.

Copyright (c) 1995 by Ken Henderson.

A couple of caveats:

1)  This magic is performed through the use of temporary views, so

	a)  Since some platforms, like Sybase, don't support 'temporary' views,
        I have to construct a temp name and both create and drop the view.  I use
        the date and time to create a name, so name collisions with other users
        are remotely possible.  See the source code.  You can handle the exception
        that is raised, if this happens, and simply re-issue the Open -- it's up
        to you.

	b)  Your users will need permission to create views, obviously

	c)  You *must* explicitly close LiveQuery components -- we drop the temporary
        view during the close.  Failing to do so will litter your server with lots
        of temp views.

	d)  Because it create views, the component is only usable on servers that
        support views, i.e. remote servers -- you can't use it with dBase and Paradox
        tables.

On the positive side, you can:

1)  Use any syntax your server supports for updatable views, including:

	a)  as many tables as you want via joins

	b)  where and having clauses

This puts all the burden on the server, where, IMHO, it belongs.  It also may mean
that the SQL you execute will be compiled ahead of time, which should make it execute
more efficiently.  If your server doesn't like an update you try to perform,
obviously an exception will be raised.

}

unit Liveqry;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, DbiTypes, DbiProcs, DbiErrs;

const
  DefaultCreateViewSQL = 'CREATE VIEW %s AS ';
  DefaultDropViewSQL = 'DROP VIEW %s';
  DefaultTableNameFormat = 'dbo.%s';

type
  TLiveQuery = class(TTable)
  private
    { Private declarations }
    FCreateViewSQL : String;
    FDropViewSQL : String;
    FTableNameFormat : TFileName;
    FSQL : TStrings;
    procedure SetQuery(Value: TStrings);
  protected
    { Protected declarations }
    procedure DoBeforeOpen; override;
    procedure DoAfterClose; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property CreateViewSQL : String read FCreateViewSQL write FCreateViewSQL;
    property DropViewSQL : String read FDropViewSQL write FDropViewSQL;
    property SQL : TStrings read FSQL write SetQuery;
    property TableNameFormat : TFileName read FTableNameFormat write FTableNameFormat;
  end;

procedure Register;

implementation

constructor TLiveQuery.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSQL := TStringList.Create;
  FCreateViewSQL := DefaultCreateViewSQL;
  FDropViewSQL := DefaultDropViewSQL;
  FTableNameFormat := DefaultTableNameFormat;
end;

destructor TLiveQuery.Destroy;
begin
  SQL.Free;
  inherited Destroy;
end;

procedure TLiveQuery.SetQuery(Value: TStrings);
begin
  CheckInActive;
  SQL.Assign(Value);
end;

procedure TLiveQuery.DoBeforeOpen;
var
  TemporaryDB : TDatabase;
  WorkSQL : TStrings;
begin
  inherited DoBeforeOpen;
  WorkSQL := TStringList.Create;
  WorkSQL.AddStrings(SQL);
  TableName:=Format(TableNameFormat,['_tv_'+FormatDateTime('yymmddhhnnss',Now)]);
  WorkSQL.Insert(0,Format(CreateViewSQL,[TableName]));
  TemporaryDB:=Session.OpenDatabase(DatabaseName);
  If (TemporaryDB<>nil) then
    try
      If (TemporaryDB.IsSQLBased) then begin
        If (DbiQExecDirect(TemporaryDB.Handle,qrylangSQL,WorkSQL.GetText,nil)<>DBIERR_NONE) then
           raise EDatabaseError.Create('Error creating temporary view');
      end else raise EDatabaseError.Create('Cannot use this component with local tables')
    finally
      Session.CloseDatabase(TemporaryDB);
      WorkSQL.Free;
    end;
end;

procedure TLiveQuery.DoAfterClose;
var
  TemporaryDB : TDatabase;
  WorkSQL : TStrings;
begin
  WorkSQL:=TStringList.Create;
  WorkSQL.Add(Format(DropViewSQL,[TableName]));
  TemporaryDB:=Session.OpenDatabase(DatabaseName);
  If (TemporaryDB<>nil) then begin
    try
      If (TemporaryDB.IsSQLBased) then begin
        If (DbiQExecDirect(TemporaryDB.Handle,qrylangSQL,WorkSQL.GetText,nil)<>DBIERR_NONE) then
           raise EDatabaseError.Create('Error dropping temporary view');
      end else raise EDatabaseError.Create('Cannot use this component with local tables')

    finally
      Session.CloseDatabase(TemporaryDB);
      WorkSQL.Free;
    end;
    inherited DoAfterClose;
  end;
end;

procedure Register;
begin
  RegisterComponents('Data Access', [TLiveQuery]);
end;

end.
